package org.framework.lazy.cloud.network.heartbeat.protocol.context;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioDatagramChannel;
import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import org.framework.lazy.cloud.network.heartbeat.common.adapter.ChannelTypeAdapter;
import org.framework.lazy.cloud.network.heartbeat.common.advanced.HandleChannelTypeAdvanced;
import org.framework.lazy.cloud.network.heartbeat.common.context.SocketApplicationListener;
import org.framework.lazy.cloud.network.heartbeat.protocol.handler.NettyUdpProxyHandler;
import org.framework.lazy.cloud.network.heartbeat.protocol.properties.ProtocolProxyProperties;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@Component
public class NettyUdpProxySocketApplicationListener implements SocketApplicationListener {

    private final EventLoopGroup bossGroup = new NioEventLoopGroup();
    private ChannelFuture channelFuture;
    private final ProtocolProxyProperties protocolProxyProperties;
    private final List<HandleChannelTypeAdvanced> handleChannelTypeAdvancedList;

    public NettyUdpProxySocketApplicationListener(ProtocolProxyProperties protocolProxyProperties,
                                                  List<HandleChannelTypeAdvanced> handleChannelTypeAdvancedList) {
        this.protocolProxyProperties = protocolProxyProperties;
        this.handleChannelTypeAdvancedList = handleChannelTypeAdvancedList;
    }


    /**
     * 运行
     *
     * @throws InterruptedException
     */
    @Override
    public void doRunning() throws Exception {
        try {
            ChannelTypeAdapter channelTypeAdapter = new ChannelTypeAdapter(handleChannelTypeAdvancedList);
            NettyUdpProxyHandler nettyUdpProxyHandler = new NettyUdpProxyHandler(channelTypeAdapter);// 通道业务处理

            ProtocolProxyProperties.UdpProtocolProxy udpProtocolProxy = protocolProxyProperties.getUdpProtocolProxy();
            Integer udpProtocolProxyPort = udpProtocolProxy.getPort();
            Bootstrap b = new Bootstrap();
            b.group(bossGroup)
                    .channel(NioDatagramChannel.class)
                    // 给服务端channel设置属性
                    // 设置读缓冲区为2M
                    .option(ChannelOption.SO_RCVBUF, 2048 * 1024)
                    // 设置写缓冲区为1M
                    .option(ChannelOption.SO_SNDBUF, 1024 * 1024)

                    .option(ChannelOption.SO_KEEPALIVE, true)
//                    .childOption(ChannelOption.TCP_NODELAY, false)
                    .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 1000 * 60)//连接超时时间设置为 60 秒
//                    .childOption(ChannelOption.RCVBUF_ALLOCATOR, new NettyRecvByteBufAllocator(1024 * 1024))//用于Channel分配接受Buffer的分配器 默认AdaptiveRecvByteBufAllocator.DEFAULT
                    .option(ChannelOption.WRITE_BUFFER_WATER_MARK, new WriteBufferWaterMark(1024 * 1024, 1024 * 1024 * 2))

                    .handler(nettyUdpProxyHandler);
            channelFuture = b.bind(udpProtocolProxyPort).sync();

            channelFuture.addListener((ChannelFutureListener) channelFuture -> {
                // 服务器已启动
                log.info("UDP 协议代理 服务器启动成功【{}】",udpProtocolProxyPort);
            });
            channelFuture.channel().closeFuture().sync();
        } catch (Exception e) {
            log.error("启动UDP 协议代理 失败", e);
        } finally {
            destroy();
            // 服务器已关闭
            log.warn("UDP 协议代理 服务关闭");
        }
    }

    @PreDestroy
    @Override
    public void destroy() throws Exception {
        if (channelFuture != null) {
            channelFuture.channel().close().syncUninterruptibly();
        }
        if (!bossGroup.isShutdown()) {
            bossGroup.shutdownGracefully();
        }
    }
}